var margin = {top: 10, right: 44, bottom: 50, left: 58},
    width = 800 - margin.left - margin.right,
    height = 420 - margin.top - margin.bottom;

var x = d3.scaleLog()
    .domain([100, 1e6])
    .range([0, width]);

var y = d3.scaleLog()
    .domain([150, 3000])
    .range([height, 16]);

var colorind = (_,i) => d3.schemeCategory10[[1,0,4,2,3][i]];

var xAxis = d3.axisBottom()
    .scale(x)
    .ticks(5, n=>n<1e4 ? n : n.toExponential().replace("+",""));

var yAxis = d3.axisLeft()
    .scale(y)
    .ticks(undefined, "1")
    .tickValues([150,500,1000,1500,2000,2500,3000,4000,5000,6000]);

var svg = d3.select("#svg").append("g")
    .attr("transform", "scale(2)translate(" + margin.left + "," + margin.top + ")");

svg.append("defs").append("clipPath")
    .attr("id", "clip")
  .append("rect")
    .attr("width", width)
    .attr("height", height);

svg.append("g")
    .attr("class", "x axis")
    .attr("transform", "translate(0," + height + ")")
    .call(xAxis);

var yax = svg.append("g")
    .attr("class", "y axis")
    .call(yAxis);

svg.append("text")
    .attrs({x: width/2, y: height+margin.bottom-14})
    .styles({"text-anchor":"middle", "font-size":"14px"})
    .text("Length of string");
svg.append("text")
    .attrs({"transform":"rotate(-90)", x:20-height/2, y:-margin.left, dy:"1em"})
    .styles({"text-anchor":"middle", "font-size":"14px"})
    .text("Speed (MB/s)");

var names = ["C", "Dyalog 16.0", "BMI2/SSE4.1 prototype", "Dyalog 17.0", "Sparse"],
    n = names.length,
    data = [
        [   100, 1699.58, 91.0222, 136.533, 131.072, 114.975],
        [   300, 1677.75, 223.418, 409.6  , 378.092, 322.308],
        [  1000, 1430.51, 390.095, 840.205, 885.622, 1008.25],
        [  3000, 1356.38, 534.261, 1638.4 , 1694.9 , 2286.14],
        [ 10000, 1359.29, 611.343, 2540.16, 2952.07, 4070.56],
        [ 30000, 1377.42, 658.874, 2857.67, 3600.88, 5093.47],
        [100000, 1382.33, 671.475, 3117.79, 3810.23, 5804.78],
        [300000, 1361.61, 678.708, 3137.7 , 3850.53, 5582.28],
        [   1e6, 1359.83, 678.483, 3068.45, 3880.17, 5247.08]
    ],
    series = names.map((_,i) => data.map(d => [d[0], d[i+1]])),
    paths, legend;

var paths = svg.append("g")
    .attrs({"class":"lines", "clip-path":"url(#clip)"})
    .selectAll("path").data(series)
    .enter().append("path")
    .style("stroke", colorind)
    .attr("d", d3.line().x(d=>x(d[0])).y(d=>y(d[1])));
  
    var offset = [-10, 10, -13, -10, -14];
var legend = svg.append("g").attr("class", "legend")
    .selectAll("text").data(names)
    .enter().append("text")
    .attrs({dy: ".38em", x: width-3, "font-size":"16px",
           y: (d,i)=>offset[i]+(s=>y(s[s.length-1][1]))(series[i])})
    .styles({"font-size": "16px", "text-anchor": "end"})
    .text(function(d) { return d; });

[paths, legend].forEach(function (e) {
    e.filter((_,j)=>j>2).style("opacity", 0);
});

function fadein(i) {
    [paths, legend].forEach(function (e) {
        e.filter((_,j)=>j===i)
            .transition().duration(800)
            .style("opacity", 1);
    });
}


var step = 0;
document.onclick = function() {
    switch (step++) {
        case 0:
            y.domain([300, 6000]);
            yax   .transition().duration(1400).ease(d3.easeSinIn).call(yAxis);
            paths .transition().duration(1400).ease(d3.easeSinIn) .attr("d", d3.line().x(d=>x(d[0])).y(d=>y(d[1])));
            legend.transition().duration(1400).ease(d3.easeSinIn)
                .attr("y", (d,i)=>offset[i]+(s=>y(s[s.length-1][1]))(series[i]));
            break;
        case 1: fadein(3); break;
        case 2: fadein(4); break;
    }
}
